@@ -130,17 +130,10 @@ static const char *scale_size(unsigned long size, int flags, struct commandline_
 			snprintf(buf, sizeof(buf), "%lld", ((long long int)size) * 1024);
 			return buf;
 		}
-		if (args.exponent == 2) {
-			if (!(flags & FREE_SI))
-				snprintf(buf, sizeof(buf), "%ld", size);
-			else
-				snprintf(buf, sizeof(buf), "%ld", (long int)(size / 0.9765625));
-			return buf;
-		}
-		if (args.exponent > 2) {
+		if (args.exponent > 1) {
 			/* In desired scale. */
 			snprintf(buf, sizeof(buf), "%ld",
-				 (long int)(size / power(base, args.exponent - 2))
+                 (long int)((size * 1024) / power(base, args.exponent-1))
 			    );
 			return buf;
 		}
@@ -148,36 +141,36 @@ static const char *scale_size(unsigned long size, int flags, struct commandline_
 
 	/* human readable output */
 	up = nextup;
-	for (i = 1; up[0] != '0'; i++, up++) {
+	for (i = 1; up[0] != 0; i++, up++) {
 		switch (i) {
 		case 1:
 			if (4 >= snprintf(buf, sizeof(buf), "%ld%c", (long)size * 1024, *up))
 				return buf;
 			break;
 		case 2:
-
-			if (!(flags & FREE_SI)) {
-				if (4 >= snprintf(buf, sizeof(buf), "%ld%c", size, *up))
-					return buf;
-			} else {
-				if (4 >=
-				    snprintf(buf, sizeof(buf), "%ld%c",
-					     (long)(size / 0.9765625), *up))
-					return buf;
-			}
-			break;
 		case 3:
 		case 4:
 		case 5:
 		case 6:
-			if (4 >=
-			    snprintf(buf, sizeof(buf), "%.1f%c",
-				     (float)(size / power(base, i - 2)), *up))
-				return buf;
-			if (4 >=
-			    snprintf(buf, sizeof(buf), "%ld%c",
-				     (long)(size / power(base, i - 2)), *up))
-				return buf;
+			if (!(flags & FREE_SI)) {
+				if (5 >=
+				    snprintf(buf, sizeof(buf), "%.1f%ci",
+					     (float)((size / 1024) * base / power(base, i - 2)), *up))
+					return buf;
+				if (5 >=
+				    snprintf(buf, sizeof(buf), "%ld%ci",
+					     (long)((size / 1024) * base / power(base, i - 2)), *up))
+					return buf;
+			} else {
+				if (4 >=
+				    snprintf(buf, sizeof(buf), "%.1f%c",
+					     (float)((size / 1024) * base / power(base, i - 2)), *up))
+					return buf;
+				if (4 >=
+				    snprintf(buf, sizeof(buf), "%ld%c",
+					     (long)((size / 1024) * base / power(base, i - 2)), *up))
+					return buf;
+			}
 			break;
 		case 7:
 			break;
@@ -202,7 +195,6 @@ static void check_unit_set(int *unit_set)
 int main(int argc, char **argv)
 {
 	int c, flags = 0, unit_set = 0;
-	char *endptr;
 	struct commandline_arguments args;
 
 	/*
@@ -324,9 +316,7 @@ int main(int argc, char **argv)
 		case 's':
 			flags |= FREE_REPEAT;
 			errno = 0;
-			args.repeat_interval = (1000000 * strtof(optarg, &endptr));
-			if (errno || optarg == endptr || (endptr && *endptr))
-				xerrx(EXIT_FAILURE, _("seconds argument `%s' failed"), optarg);
+            args.repeat_interval = (1000000 * strtod_nol_or_err(optarg, "seconds argument failed"));
 			if (args.repeat_interval < 1)
 				xerrx(EXIT_FAILURE,
 				     _("seconds argument `%s' is not positive number"), optarg);
