@@ -46,13 +46,13 @@
 
 #include "c.h"
 #include "fileutils.h"
+#include "nsutils.h"
 #include "nls.h"
 #include "xalloc.h"
 #include "proc/readproc.h"
 #include "proc/sig.h"
 #include "proc/devname.h"
 #include "proc/sysinfo.h"
-#include "proc/version.h" /* procps_version */
 
 static int i_am_pkill = 0;
 
@@ -76,6 +76,7 @@ static int opt_lock = 0;
 static int opt_case = 0;
 static int opt_echo = 0;
 static int opt_threads = 0;
+static pid_t opt_ns_pid = 0;
 
 static const char *opt_delim = "\n";
 static struct el *opt_pgrp = NULL;
@@ -86,9 +87,13 @@ static struct el *opt_sid = NULL;
 static struct el *opt_term = NULL;
 static struct el *opt_euid = NULL;
 static struct el *opt_ruid = NULL;
+static struct el *opt_nslist = NULL;
 static char *opt_pattern = NULL;
 static char *opt_pidfile = NULL;
 
+/* by default, all namespaces will be checked */
+static int ns_flags = 0x3f;
+
 static int __attribute__ ((__noreturn__)) usage(int opt)
 {
 	int err = (opt == '?');
@@ -98,52 +103,64 @@ static int __attribute__ ((__noreturn__)) usage(int opt)
 	fprintf(fp, _(" %s [options] <pattern>\n"), program_invocation_short_name);
 	fputs(USAGE_OPTIONS, fp);
 	if (i_am_pkill == 0) {
-		fputs(_(" -d, --delimiter <string>  specify output delimiter\n"
-			" -l, --list-name           list PID and process name\n"
-			" -v, --inverse             negates the matching\n"
-			" -w, --lightweight         list all TID\n"
-		), fp);
+		fputs(_(" -d, --delimiter <string>  specify output delimiter\n"),fp);
+		fputs(_(" -l, --list-name           list PID and process name\n"),fp);
+		fputs(_(" -a, --list-full           list PID and full command line\n"),fp);
+		fputs(_(" -v, --inverse             negates the matching\n"),fp);
+		fputs(_(" -w, --lightweight         list all TID\n"), fp);
 	}
 	if (i_am_pkill == 1) {
-		fputs(_(" -<sig>, --signal <sig>    signal to send (either number or name)\n"
-			" -e, --echo                display what is killed\n"), fp);
+		fputs(_(" -<sig>, --signal <sig>    signal to send (either number or name)\n"), fp);
+		fputs(_(" -e, --echo                display what is killed\n"), fp);
 	}
-	fputs(_(" -c, --count               count of matching processes\n"
-		" -f, --full                use full process name to match\n"
-		" -g, --pgroup <id,...>     match listed process group IDs\n"
-		" -G, --group <gid,...>     match real group IDs\n"
-		" -n, --newest              select most recently started\n"
-		" -o, --oldest              select least recently started\n"
-		" -P, --parent <ppid,...>   match only child processes of the given parent\n"
-		" -s, --session <sid,...>   match session IDs\n"
-		" -t, --terminal <tty,...>  match by controlling terminal\n"
-		" -u, --euid <id,...>       match by effective IDs\n"
-		" -U, --uid <id,...>        match by real IDs\n"
-		" -x, --exact               match exactly with the command name\n"
-		" -F, --pidfile <file>      read PIDs from file\n"
-		" -L, --logpidfile          fail if PID file is not locked\n"), fp);
+	fputs(_(" -c, --count               count of matching processes\n"), fp);
+	fputs(_(" -f, --full                use full process name to match\n"), fp);
+	fputs(_(" -g, --pgroup <PGID,...>   match listed process group IDs\n"), fp);
+	fputs(_(" -G, --group <GID,...>     match real group IDs\n"), fp);
+	fputs(_(" -i, --ignore-case         match case insensitively\n"), fp);
+	fputs(_(" -n, --newest              select most recently started\n"), fp);
+	fputs(_(" -o, --oldest              select least recently started\n"), fp);
+	fputs(_(" -P, --parent <PPID,...>   match only child processes of the given parent\n"), fp);
+	fputs(_(" -s, --session <SID,...>   match session IDs\n"), fp);
+	fputs(_(" -t, --terminal <tty,...>  match by controlling terminal\n"), fp);
+	fputs(_(" -u, --euid <ID,...>       match by effective IDs\n"), fp);
+	fputs(_(" -U, --uid <ID,...>        match by real IDs\n"), fp);
+	fputs(_(" -x, --exact               match exactly with the command name\n"), fp);
+	fputs(_(" -F, --pidfile <file>      read PIDs from file\n"), fp);
+	fputs(_(" -L, --logpidfile          fail if PID file is not locked\n"), fp);
+	fputs(_(" --ns <PID>                match the processes that belong to the same\n"
+		"                           namespace as <pid> or 0 for all namespaces\n"), fp);
+	fputs(_(" --nslist <ns,...>         list which namespaces will be considered for\n"
+		"                           the --ns option.\n"
+		"                           Available namespaces: ipc, mnt, net, pid, user, uts\n"), fp);
 	fputs(USAGE_SEPARATOR, fp);
 	fputs(USAGE_HELP, fp);
 	fputs(USAGE_VERSION, fp);
 	fprintf(fp, USAGE_MAN_TAIL("pgrep(1)"));
 
-	exit(fp == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
+	exit(fp == stderr ? EXIT_USAGE : EXIT_SUCCESS);
 }
 
 static struct el *split_list (const char *restrict str, int (*convert)(const char *, struct el *))
 {
-	char *copy = xstrdup (str);
-	char *ptr = copy;
+	char *copy;
+	char *ptr;
 	char *sep_pos;
 	int i = 0;
 	int size = 0;
 	struct el *list = NULL;
 
+	if (str[0] == '\0')
+		return NULL;
+
+	copy = xstrdup (str);
+	ptr = copy;
+
 	do {
 		if (i == size) {
 			size = size * 5 / 4 + 4;
 			/* add 1 because slot zero is a count */
-			list = xrealloc (list, 1 + size * sizeof *list);
+			list = xrealloc (list, (1 + size) * sizeof *list);
 		}
 		sep_pos = strchr (ptr, ',');
 		if (sep_pos)
@@ -320,6 +337,20 @@ static int conv_str (const char *restrict name, struct el *restrict e)
 }
 
 
+static int conv_ns (const char *restrict name, struct el *restrict e)
+{
+	int rc = conv_str(name, e);
+	int id;
+
+	ns_flags = 0;
+	id = get_ns_id(name);
+	if (id == -1)
+		return 0;
+	ns_flags |= (1 << id);
+
+	return rc;
+}
+
 static int match_numlist (long value, const struct el *restrict list)
 {
 	int found = 0;
@@ -350,6 +381,21 @@ static int match_strlist (const char *restrict value, const struct el *restrict
 	return found;
 }
 
+static int match_ns (const proc_t *task, const proc_t *ns_task)
+{
+	int found = 1;
+	int i;
+
+	for (i = 0; i < NUM_NS; i++) {
+		if (ns_flags & (1 << i)) {
+			if (task->ns[i] != ns_task->ns[i])
+				found = 0;
+		}
+	}
+
+	return found;
+}
+
 static void output_numlist (const struct el *restrict list, int num)
 {
 	int i;
@@ -378,7 +424,7 @@ static PROCTAB *do_openproc (void)
 	PROCTAB *ptp;
 	int flags = 0;
 
-	if (opt_pattern || opt_full)
+	if (opt_pattern || opt_full || opt_longlong)
 		flags |= PROC_FILLCOM;
 	if (opt_ruid || opt_rgid)
 		flags |= PROC_FILLSTATUS;
@@ -386,6 +432,8 @@ static PROCTAB *do_openproc (void)
 		flags |= PROC_FILLSTAT;
 	if (!(flags & PROC_FILLSTAT))
 		flags |= PROC_FILLSTATUS;  /* FIXME: need one, and PROC_FILLANY broken */
+	if (opt_ns_pid)
+		flags |= PROC_FILLNS;
 	if (opt_euid && !opt_negate) {
 		int num = opt_euid[0].num;
 		int i = num;
@@ -419,6 +467,9 @@ static regex_t * do_regcomp (void)
 		}
 
 		re_err = regcomp (preg, re, REG_EXTENDED | REG_NOSUB | opt_case);
+
+		if (opt_exact) free(re);
+
 		if (re_err) {
 			regerror (re_err, preg, errbuf, sizeof(errbuf));
 			fputs(errbuf,stderr);
@@ -442,6 +493,7 @@ static struct el * select_procs (int *num)
 	char cmdline[CMDSTRSIZE];
 	char cmdsearch[CMDSTRSIZE];
 	char cmdoutput[CMDSTRSIZE];
+	proc_t ns_task;
 
 	ptp = do_openproc();
 	preg = do_regcomp();
@@ -451,6 +503,11 @@ static struct el * select_procs (int *num)
 
 	if (opt_newest) saved_pid = 0;
 	if (opt_oldest) saved_pid = INT_MAX;
+	if (opt_ns_pid && ns_read(opt_ns_pid, &ns_task)) {
+		fputs(_("Error reading reference namespace information\n"),
+		      stderr);
+		exit (EXIT_FATAL);
+	}
 
 	memset(&task, 0, sizeof (task));
 	while(readproc(ptp, &task)) {
@@ -476,6 +533,8 @@ static struct el * select_procs (int *num)
 			match = 0;
 		else if (opt_sid && ! match_numlist (task.session, opt_sid))
 			match = 0;
+		else if (opt_ns_pid && ! match_ns (&task, &ns_task))
+			match = 0;
 		else if (opt_term) {
 			if (task.tty == 0) {
 				match = 0;
@@ -486,7 +545,7 @@ static struct el * select_procs (int *num)
 				match = match_strlist (tty, opt_term);
 			}
 		}
-		if (task.cmdline && (opt_longlong || opt_full) && match && opt_pattern) {
+		if (task.cmdline && (opt_longlong || opt_full) ) {
 			int i = 0;
 			int bytes = sizeof (cmdline) - 1;
 
@@ -570,7 +629,7 @@ static struct el * select_procs (int *num)
 						if (list == NULL)
 							exit (EXIT_FATAL);
 					}
-					if (opt_long) {
+					if (opt_long || opt_longlong) {
 						list[matches].str = xstrdup (cmdoutput);
 						list[matches++].num = subtask.XXXID;
 					} else {
@@ -592,6 +651,11 @@ static struct el * select_procs (int *num)
 	}
 	closeproc (ptp);
 	*num = matches;
+
+	if ((!matches) && (!opt_full) && (strlen(opt_pattern) > 15))
+		xwarnx(_("pattern that searches for process name longer than 15 characters will result in zero matches\n"
+				 "Try `%s -f' option to match against the complete command line."),
+				 program_invocation_short_name);
 	return list;
 }
 
@@ -602,8 +666,6 @@ static int signal_option(int *argc, char **argv)
 	for (i = 1; i < *argc; i++) {
 		if (argv[i][0] == '-') {
 			sig = signal_name_to_number(argv[i] + 1);
-			if (sig == -1 && isdigit(argv[i][1]))
-				sig = atoi(argv[i] + 1);
 			if (-1 < sig) {
 				memmove(argv + i, argv + i + 1,
 					sizeof(char *) * (*argc - i));
@@ -617,12 +679,14 @@ static int signal_option(int *argc, char **argv)
 
 static void parse_opts (int argc, char **argv)
 {
-	char opts[32] = "";
+	char opts[64] = "";
 	int opt;
 	int criteria_count = 0;
 
 	enum {
-		SIGNAL_OPTION = CHAR_MAX + 1
+		SIGNAL_OPTION = CHAR_MAX + 1,
+		NS_OPTION,
+		NSLIST_OPTION,
 	};
 	static const struct option longopts[] = {
 		{"signal", required_argument, NULL, SIGNAL_OPTION},
@@ -633,6 +697,7 @@ static void parse_opts (int argc, char **argv)
 		{"full", no_argument, NULL, 'f'},
 		{"pgroup", required_argument, NULL, 'g'},
 		{"group", required_argument, NULL, 'G'},
+		{"ignore-case", no_argument, NULL, 'i'},
 		{"newest", no_argument, NULL, 'n'},
 		{"oldest", no_argument, NULL, 'o'},
 		{"parent", required_argument, NULL, 'P'},
@@ -646,6 +711,8 @@ static void parse_opts (int argc, char **argv)
 		{"pidfile", required_argument, NULL, 'F'},
 		{"logpidfile", no_argument, NULL, 'L'},
 		{"echo", no_argument, NULL, 'e'},
+		{"ns", required_argument, NULL, NS_OPTION},
+		{"nslist", required_argument, NULL, NSLIST_OPTION},
 		{"help", no_argument, NULL, 'h'},
 		{"version", no_argument, NULL, 'V'},
 		{NULL, 0, NULL, 0}
@@ -664,7 +731,7 @@ static void parse_opts (int argc, char **argv)
 		strcat (opts, "lad:vw");
 	}
 
-	strcat (opts, "LF:cfnoxP:g:s:u:U:G:t:?Vh");
+	strcat (opts, "LF:cfinoxP:g:s:u:U:G:t:?Vh");
 
 	while ((opt = getopt_long (argc, argv, opts, longopts, NULL)) != -1) {
 		switch (opt) {
@@ -685,7 +752,7 @@ static void parse_opts (int argc, char **argv)
 		case 'G':   /* Solaris: match rgid/rgroup */
 			opt_rgid = split_list (optarg, conv_gid);
 			if (opt_rgid == NULL)
-				usage (opt);
+				usage ('?');
 			++criteria_count;
 			break;
 /*		case 'I':   / * FreeBSD: require confirmation before killing * /
@@ -702,7 +769,7 @@ static void parse_opts (int argc, char **argv)
 		case 'P':   /* Solaris: match by PPID */
 			opt_ppid = split_list (optarg, conv_num);
 			if (opt_ppid == NULL)
-				usage (opt);
+				usage ('?');
 			++criteria_count;
 			break;
 /*		case 'S':   / * FreeBSD: don't ignore the built-in kernel tasks * /
@@ -712,7 +779,7 @@ static void parse_opts (int argc, char **argv)
 		case 'U':   /* Solaris: match by ruid/rgroup */
 			opt_ruid = split_list (optarg, conv_uid);
 			if (opt_ruid == NULL)
-				usage (opt);
+				usage ('?');
 			++criteria_count;
 			break;
 		case 'V':
@@ -732,14 +799,14 @@ static void parse_opts (int argc, char **argv)
 		case 'g':   /* Solaris: match pgrp */
 			opt_pgrp = split_list (optarg, conv_pgrp);
 			if (opt_pgrp == NULL)
-				usage (opt);
+				usage ('?');
 			++criteria_count;
 			break;
-/*		case 'i':   / * FreeBSD: ignore case. OpenBSD: withdrawn. See -I. This sucks. * /
- *			if (opt_case)
- *				usage (opt);
- *			opt_case = REG_ICASE;
- *			break; */
+		case 'i':   /* FreeBSD: ignore case. OpenBSD: withdrawn. See -I. This sucks. */
+			if (opt_case)
+				usage (opt);
+			opt_case = REG_ICASE;
+			break;
 /*		case 'j':   / * FreeBSD: restricted to the given jail ID * /
  *			break; */
 		case 'l':   /* Solaris: long output format (pgrep only) Should require -f for beyond argv[0] maybe? */
@@ -750,37 +817,37 @@ static void parse_opts (int argc, char **argv)
 			break;
 		case 'n':   /* Solaris: match only the newest */
 			if (opt_oldest|opt_negate|opt_newest)
-				usage (opt);
+				usage ('?');
 			opt_newest = 1;
 			++criteria_count;
 			break;
 		case 'o':   /* Solaris: match only the oldest */
 			if (opt_oldest|opt_negate|opt_newest)
-				usage (opt);
+				usage ('?');
 			opt_oldest = 1;
 			++criteria_count;
 			break;
 		case 's':   /* Solaris: match by session ID -- zero means self */
 			opt_sid = split_list (optarg, conv_sid);
 			if (opt_sid == NULL)
-				usage (opt);
+				usage ('?');
 			++criteria_count;
 			break;
 		case 't':   /* Solaris: match by tty */
 			opt_term = split_list (optarg, conv_str);
 			if (opt_term == NULL)
-				usage (opt);
+				usage ('?');
 			++criteria_count;
 			break;
 		case 'u':   /* Solaris: match by euid/egroup */
 			opt_euid = split_list (optarg, conv_uid);
 			if (opt_euid == NULL)
-				usage (opt);
+				usage ('?');
 			++criteria_count;
 			break;
 		case 'v':   /* Solaris: as in grep, invert the matching (uh... applied after selection I think) */
 			if (opt_oldest|opt_negate|opt_newest)
-				usage (opt);
+				usage ('?');
 			opt_negate = 1;
 			break;
 		case 'w':   // Linux: show threads (lightweight process) too
@@ -792,17 +859,24 @@ static void parse_opts (int argc, char **argv)
 			break;
 /*		case 'z':   / * Solaris: match by zone ID * /
  *			break; */
-		case 'h':
-			usage (opt);
+		case NS_OPTION:
+			opt_ns_pid = atoi(optarg);
+			++criteria_count;
 			break;
+		case NSLIST_OPTION:
+			opt_nslist = split_list (optarg, conv_ns);
+			if (opt_nslist == NULL)
+				usage ('?');
+			break;
+		case 'h':
 		case '?':
-			usage (optopt ? optopt : opt);
+			usage (opt);
 			break;
 		}
 	}
 
 	if(opt_lock && !opt_pidfile)
-		xerrx(EXIT_FAILURE, _("-L without -F makes no sense\n"
+		xerrx(EXIT_USAGE, _("-L without -F makes no sense\n"
 				     "Try `%s --help' for more information."),
 				     program_invocation_short_name);
 
@@ -817,11 +891,11 @@ static void parse_opts (int argc, char **argv)
 	if (argc - optind == 1)
 		opt_pattern = argv[optind];
 	else if (argc - optind > 1)
-		xerrx(EXIT_FAILURE, _("only one pattern can be provided\n"
+		xerrx(EXIT_USAGE, _("only one pattern can be provided\n"
 				     "Try `%s --help' for more information."),
 				     program_invocation_short_name);
 	else if (criteria_count == 0)
-		xerrx(EXIT_FAILURE, _("no matching criteria specified\n"
+		xerrx(EXIT_USAGE, _("no matching criteria specified\n"
 				     "Try `%s --help' for more information."),
 				     program_invocation_short_name);
 }
@@ -840,15 +914,18 @@ int main (int argc, char **argv)
 	textdomain(PACKAGE);
 	atexit(close_stdout);
 
+	opt_ns_pid = getpid();
 	parse_opts (argc, argv);
 
 	procs = select_procs (&num);
 	if (i_am_pkill) {
 		int i;
+        int kill_count = 0;
 		for (i = 0; i < num; i++) {
 			if (kill (procs[i].num, opt_signal) != -1) {
 				if (opt_echo)
 					printf(_("%s killed (pid %lu)\n"), procs[i].str, procs[i].num);
+                kill_count++;
 				continue;
 			}
 			if (errno==ESRCH)
@@ -858,6 +935,7 @@ int main (int argc, char **argv)
 		}
 		if (opt_count)
 			fprintf(stdout, "%d\n", num);
+        return !kill_count;
 	} else {
 		if (opt_count) {
 			fprintf(stdout, "%d\n", num);
