@@ -201,7 +201,7 @@ static void discover_shm_minor(void)
 		perror(_("shared memory detach"));
 
 out_destroy:
-	if (shmctl(shmid, IPC_RMID, NULL))
+	if (shmctl(shmid, IPC_RMID, NULL) && errno != EINVAL)
 		perror(_("shared memory remove"));
 
 	return;
@@ -314,17 +314,17 @@ static void print_extended_maps (FILE *f)
 	while (ret != NULL) {
 		/* === READ MAPPING === */
 		map_desc[0] = '\0';
-		c = '\n';
 		nfields = sscanf(mapbuf,
 				 "%"NUML"[0-9a-f]-%"NUML"[0-9a-f] "
 				 "%"DETL"s %"NUML"[0-9a-f] "
-				 "%63[0-9a-f:] %"NUML"s %127[^\n]%c",
+				 "%63[0-9a-f:] %"NUML"s %127[^\n]",
 				 start, end, perms, offset,
-				 dev, inode, map_desc, &c);
+				 dev, inode, map_desc);
 		/* Must read at least up to inode, else something has changed! */
 		if (nfields < 6)
 			xerrx(EXIT_FAILURE, _("Unknown format in smaps file!"));
 		/* If line too long we dump everything else. */
+		c = mapbuf[strlen(mapbuf) - 1];
 		while (c != '\n') {
 			ret = fgets(mapbuf, sizeof mapbuf, f);
 			c = mapbuf[strlen(mapbuf) - 1];
@@ -597,25 +597,23 @@ static int one_proc(proc_t * p)
 		/* hex values are lower case or numeric, keys are upper */
 		if (mapbuf[0] >= 'A' && mapbuf[0] <= 'Z') {
 			/* Its a key */
-			if (sscanf
-			    (mapbuf, "%20[^:]: %llu", smap_key,
-			     &smap_value) == 2) {
-				if (strncmp("Rss", smap_key, 3) == 0) {
+			if (sscanf(mapbuf, "%20[^:]: %llu", smap_key, &smap_value) == 2) {
+				if (strcmp("Rss", smap_key) == 0) {
 					rss = smap_value;
 					total_rss += smap_value;
 					continue;
 				}
-				if (strncmp("Shared_Dirty", smap_key, 12) == 0) {
+				if (strcmp("Shared_Dirty", smap_key) == 0) {
 					shared_dirty = smap_value;
 					total_shared_dirty += smap_value;
 					continue;
 				}
-				if (strncmp("Private_Dirty", smap_key, 13) == 0) {
+				if (strcmp("Private_Dirty", smap_key) == 0) {
 					private_dirty = smap_value;
 					total_private_dirty += smap_value;
 					continue;
 				}
-				if (strncmp("Swap", smap_key, 4) == 0) {
+				if (strcmp("Swap", smap_key) == 0) {
 					/*doesn't matter as long as last */
 					printf("%0*" KLF "x %*lu %*llu %*llu %*s %s\n",
 					       maxw1, start,
@@ -629,9 +627,9 @@ static int one_proc(proc_t * p)
 					diff = 0;
 					continue;
 				}
-				/* Other keys */
-				continue;
 			}
+			/* Other keys or not a key-value pair */
+			continue;
 		}
 		sscanf(mapbuf, "%" KLF "x-%" KLF "x %31s %llx %x:%x %llu", &start,
 		       &end, perms, &file_offset, &dev_major, &dev_minor,
