@@ -40,6 +40,9 @@
 #ifdef WITH_SYSTEMD
 #include <systemd/sd-login.h>
 #endif
+#ifdef WITH_ELOGIND
+#include <elogind/sd-login.h>
+#endif
 
 // sometimes it's easier to do this manually, w/o gcc helping
 #ifdef PROF
@@ -93,9 +96,9 @@ static inline void free_acquired (proc_t *p, int reuse) {
         if (p->environ)  free((void*)*p->environ);
         if (p->cmdline)  free((void*)*p->cmdline);
         if (p->cgroup)   free((void*)*p->cgroup);
+        if (p->cgname)   free(p->cgname);
         if (p->supgid)   free(p->supgid);
         if (p->supgrp)   free(p->supgrp);
-#ifdef WITH_SYSTEMD
         if (p->sd_mach)  free(p->sd_mach);
         if (p->sd_ouid)  free(p->sd_ouid);
         if (p->sd_seat)  free(p->sd_seat);
@@ -103,7 +106,6 @@ static inline void free_acquired (proc_t *p, int reuse) {
         if (p->sd_slice) free(p->sd_slice);
         if (p->sd_unit)  free(p->sd_unit);
         if (p->sd_uunit) free(p->sd_uunit);
-#endif
 #ifdef QUICK_THREADS
     }
 #endif
@@ -113,7 +115,7 @@ static inline void free_acquired (proc_t *p, int reuse) {
 ///////////////////////////////////////////////////////////////////////////
 
 typedef struct status_table_struct {
-    unsigned char name[7];        // /proc/*/status field name
+    unsigned char name[8];        // /proc/*/status field name
     unsigned char len;            // name length
 #ifdef LABEL_OFFSET
     long offset;                  // jump address offset
@@ -129,8 +131,11 @@ typedef struct status_table_struct {
 #endif
 #define NUL  {"", 0, 0},
 
+#define GPERF_TABLE_SIZE 128
+
 // Derived from:
 // gperf -7 --language=ANSI-C --key-positions=1,3,4 -C -n -c <if-not-piped>
+// ( --key-positions verified by omission & reported "Computed positions" )
 //
 // Suggested method:
 // Grep this file for "case_", then strip those down to the name.
@@ -144,8 +149,8 @@ typedef struct status_table_struct {
 // the F macro and replacing empty strings with the NUL define.
 //
 // In the status_table_struct watch out for name size (grrr, expanding)
-// and the number of entries (we mask with 63 for now). The table
-// must be padded out to 64 entries, maybe 128 in the future.
+// and the number of entries. Currently, the table is padded to 128
+// entries and we therefore mask with 127.
 
 static void status2proc(char *S, proc_t *restrict P, int is_proc){
     long Threads = 0;
@@ -155,78 +160,76 @@ static void status2proc(char *S, proc_t *restrict P, int is_proc){
   // 128 entries because we trust the kernel to use ASCII names
   static const unsigned char asso[] =
     {
-      64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-      64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-      64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-      64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-      64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-      64, 64, 64, 64, 64, 64, 64, 64, 28, 64,
-      64, 64, 64, 64, 64, 64,  8, 25, 23, 25,
-       6, 25,  0,  3, 64, 64,  3, 64, 25, 64,
-      20,  1,  1,  5,  0, 30,  0,  0, 64, 64,
-      64, 64, 64, 64, 64, 64, 64,  3, 64,  0,
-       0, 18, 64, 10, 64, 10, 64, 64, 64, 20,
-      64, 20,  0, 64, 25, 64,  3, 15, 64,  0,
-      30, 64, 64, 64, 64, 64, 64, 64
+      101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
+      101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
+      101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
+      101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
+      101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
+      101, 101, 101, 101, 101, 101, 101, 101,   6, 101,
+      101, 101, 101, 101, 101,  45,  55,  25,  31,  50,
+       50,  10,   0,  35, 101, 101,  21, 101,  30, 101,
+       20,  36,   0,   5,   0,  40,   0,   0, 101, 101,
+      101, 101, 101, 101, 101, 101, 101,  30, 101,  15,
+        0,   1, 101,  10, 101,  10, 101, 101, 101,  25,
+      101,  40,   0, 101,   0,  50,   6,  40, 101,   1,
+       35, 101, 101, 101, 101, 101, 101, 101
     };
 
-    static const status_table_struct table[] = {
+    static const status_table_struct table[GPERF_TABLE_SIZE] = {
       F(VmHWM)
-      NUL NUL
-      F(VmLck)
-      NUL
-      F(VmSwap)
-      F(VmRSS)
-      NUL
-      F(VmStk)
-      NUL
-      F(Tgid)
-      F(State)
-      NUL
-      F(VmLib)
-      NUL
-      F(VmSize)
-      F(SigQ)
-      NUL
-      F(SigIgn)
-      NUL
-      F(VmPTE)
-      F(FDSize)
-      NUL
-      F(SigBlk)
-      NUL
-      F(ShdPnd)
-      F(VmData)
-      NUL
-      F(CapInh)
-      NUL
-      F(PPid)
-      NUL NUL
-      F(CapBnd)
-      NUL
-      F(SigPnd)
-      NUL NUL
-      F(VmPeak)
-      NUL
-      F(SigCgt)
-      NUL NUL
       F(Threads)
-      NUL
-      F(CapPrm)
-      NUL NUL
-      F(Pid)
-      NUL
-      F(CapEff)
-      NUL NUL
+      NUL NUL NUL
+      F(VmRSS)
+      F(VmSwap)
+      NUL NUL NUL
+      F(Tgid)
+      F(VmStk)
+      NUL NUL NUL
+      F(VmSize)
       F(Gid)
-      NUL
-      F(VmExe)
-      NUL NUL
+      NUL NUL NUL
+      F(VmPTE)
+      F(VmPeak)
+      NUL NUL NUL
+      F(ShdPnd)
+      F(Pid)
+      NUL NUL NUL
+      F(PPid)
+      F(VmLib)
+      NUL NUL NUL
+      F(SigPnd)
+      F(VmLck)
+      NUL NUL NUL
+      F(SigCgt)
+      F(State)
+      NUL NUL NUL
+      F(CapPrm)
       F(Uid)
-      NUL
-      F(Groups)
-      NUL NUL
+      NUL NUL NUL
+      F(SigIgn)
+      F(SigQ)
+      NUL NUL NUL
+      F(RssShmem)
       F(Name)
+      NUL NUL NUL
+      F(CapInh)
+      F(VmData)
+      NUL NUL NUL
+      F(FDSize)
+      NUL NUL NUL NUL
+      F(SigBlk)
+      NUL NUL NUL NUL
+      F(CapEff)
+      NUL NUL NUL NUL
+      F(CapBnd)
+      NUL NUL NUL NUL
+      F(VmExe)
+      NUL NUL NUL NUL
+      F(Groups)
+      NUL NUL NUL NUL
+      F(RssAnon)
+      NUL NUL NUL NUL
+      F(RssFile)
     };
 
 #undef F
@@ -248,7 +251,7 @@ ENTER(0x220);
         // examine a field name (hash and compare)
     base:
         if(unlikely(!*S)) break;
-        entry = table[63 & (asso[(int)S[3]] + asso[(int)S[2]] + asso[(int)S[0]])];
+        entry = table[(GPERF_TABLE_SIZE -1) & (asso[(int)S[3]] + asso[(int)S[2]] + asso[(int)S[0]])];
         colon = strchr(S, ':');
         if(unlikely(!colon)) break;
         if(unlikely(colon[1]!='\t')) break;
@@ -361,6 +364,15 @@ ENTER(0x220);
     case_VmRSS:
         P->vm_rss = strtol(S,&S,10);
         continue;
+    case_RssAnon:       // subset of VmRSS, linux-4.5
+        P->vm_rss_anon = strtol(S,&S,10);
+        continue;
+    case_RssFile:       // subset of VmRSS, linux-4.5
+        P->vm_rss_file = strtol(S,&S,10);
+        continue;
+    case_RssShmem:      // subset of VmRSS, linux-4.5
+        P->vm_rss_shared = strtol(S,&S,10);
+        continue;
     case_VmSize:
         P->vm_size = strtol(S,&S,10);
         continue;
@@ -436,6 +448,7 @@ ENTER(0x220);
 
 LEAVE(0x220);
 }
+#undef GPERF_TABLE_SIZE
 
 static void supgrps_from_supgids (proc_t *p) {
     char *g, *s;
@@ -449,14 +462,13 @@ static void supgrps_from_supgids (proc_t *p) {
     t = 0;
     do {
         if (',' == *s) ++s;
-        g = group_from_gid((uid_t)strtol(s, &s, 10));
+        g = pwcache_get_group((uid_t)strtol(s, &s, 10));
         p->supgrp = xrealloc(p->supgrp, P_G_SZ+t+2);
         t += snprintf(p->supgrp+t, P_G_SZ+2, "%s%s", t ? "," : "", g);
     } while (*s);
 }
 
 ///////////////////////////////////////////////////////////////////////
-#ifdef OOMEM_ENABLE
 static void oomscore2proc(const char* S, proc_t *restrict P)
 {
     sscanf(S, "%d", &P->oom_score);
@@ -466,7 +478,6 @@ static void oomadj2proc(const char* S, proc_t *restrict P)
 {
     sscanf(S, "%d", &P->oom_adj);
 }
-#endif
 ///////////////////////////////////////////////////////////////////////
 
 static const char *ns_names[] = {
@@ -509,8 +520,8 @@ static void ns2proc(const char *directory, proc_t *restrict p) {
     }
 }
 
-#ifdef WITH_SYSTEMD
 static void sd2proc(proc_t *restrict p) {
+#if defined(WITH_SYSTEMD) || defined(WITH_ELOGIND)
     char buf[64];
     uid_t uid;
 
@@ -523,7 +534,6 @@ static void sd2proc(proc_t *restrict p) {
         snprintf(buf, sizeof(buf), "%d", (int)uid);
         p->sd_ouid = strdup(buf);
     }
-
     if (0 > sd_pid_get_session(p->tid, &p->sd_sess)) {
         p->sd_sess = strdup("-");
         p->sd_seat = strdup("-");
@@ -531,17 +541,22 @@ static void sd2proc(proc_t *restrict p) {
         if (0 > sd_session_get_seat(p->sd_sess, &p->sd_seat))
             p->sd_seat = strdup("-");
     }
-
     if (0 > sd_pid_get_slice(p->tid, &p->sd_slice))
         p->sd_slice = strdup("-");
-
     if (0 > sd_pid_get_unit(p->tid, &p->sd_unit))
         p->sd_unit = strdup("-");
-
     if (0 > sd_pid_get_user_unit(p->tid, &p->sd_uunit))
         p->sd_uunit = strdup("-");
-}
+#else
+    p->sd_mach  = strdup("?");
+    p->sd_ouid  = strdup("?");
+    p->sd_seat  = strdup("?");
+    p->sd_sess  = strdup("?");
+    p->sd_slice = strdup("?");
+    p->sd_unit  = strdup("?");
+    p->sd_uunit = strdup("?");
 #endif
+}
 ///////////////////////////////////////////////////////////////////////
 
 
@@ -571,11 +586,11 @@ ENTER(0x160);
        "%c "
        "%d %d %d %d %d "
        "%lu %lu %lu %lu %lu "
-       "%Lu %Lu %Lu %Lu "  /* utime stime cutime cstime */
+       "%llu %llu %llu %llu "  /* utime stime cutime cstime */
        "%ld %ld "
        "%d "
        "%ld "
-       "%Lu "  /* start_time */
+       "%llu "  /* start_time */
        "%lu "
        "%ld "
        "%lu %"KLF"u %"KLF"u %"KLF"u %"KLF"u %"KLF"u "
@@ -612,11 +627,9 @@ LEAVE(0x160);
 /////////////////////////////////////////////////////////////////////////
 
 static void statm2proc(const char* s, proc_t *restrict P) {
-    int num;
-    num = sscanf(s, "%ld %ld %ld %ld %ld %ld %ld",
+    sscanf(s, "%ld %ld %ld %ld %ld %ld %ld",
 	   &P->size, &P->resident, &P->share,
 	   &P->trs, &P->lrs, &P->drs, &P->dt);
-/*    fprintf(stderr, "statm2proc converted %d fields.\n",num); */
 }
 
 static int file2str(const char *directory, const char *what, struct utlbuf_s *ub) {
@@ -762,7 +775,7 @@ static char** vectorize_this_str (const char* src) {
     // the data into a single string represented as a single vector.
 static void fill_cgroup_cvt (const char* directory, proc_t *restrict p) {
  #define vMAX ( MAX_BUFSZ - (int)(dst - dst_buffer) )
-    char *src, *dst, *grp, *eob;
+    char *src, *dst, *grp, *eob, *name;
     int tot, x, whackable_int = MAX_BUFSZ;
 
     *(dst = dst_buffer) = '\0';                  // empty destination
@@ -779,6 +792,10 @@ static void fill_cgroup_cvt (const char* directory, proc_t *restrict p) {
         dst += escape_str(dst, grp, vMAX, &whackable_int);
     }
     p->cgroup = vectorize_this_str(dst_buffer[0] ? dst_buffer : "-");
+
+    name = strstr(p->cgroup[0], ":name=");
+    if (name && *(name+6)) name += 6; else name = p->cgroup[0];
+    p->cgname = strdup(name);
  #undef vMAX
 }
 
@@ -939,21 +956,21 @@ static proc_t* simple_readproc(PROCTAB *restrict const PT, proc_t *restrict cons
 
     /* some number->text resolving which is time consuming */
     if (flags & PROC_FILLUSR){
-        memcpy(p->euser, user_from_uid(p->euid), sizeof p->euser);
+        memcpy(p->euser, pwcache_get_user(p->euid), sizeof p->euser);
         if(flags & PROC_FILLSTATUS) {
-            memcpy(p->ruser, user_from_uid(p->ruid), sizeof p->ruser);
-            memcpy(p->suser, user_from_uid(p->suid), sizeof p->suser);
-            memcpy(p->fuser, user_from_uid(p->fuid), sizeof p->fuser);
+            memcpy(p->ruser, pwcache_get_user(p->ruid), sizeof p->ruser);
+            memcpy(p->suser, pwcache_get_user(p->suid), sizeof p->suser);
+            memcpy(p->fuser, pwcache_get_user(p->fuid), sizeof p->fuser);
         }
     }
 
     /* some number->text resolving which is time consuming */
     if (flags & PROC_FILLGRP){
-        memcpy(p->egroup, group_from_gid(p->egid), sizeof p->egroup);
+        memcpy(p->egroup, pwcache_get_group(p->egid), sizeof p->egroup);
         if(flags & PROC_FILLSTATUS) {
-            memcpy(p->rgroup, group_from_gid(p->rgid), sizeof p->rgroup);
-            memcpy(p->sgroup, group_from_gid(p->sgid), sizeof p->sgroup);
-            memcpy(p->fgroup, group_from_gid(p->fgid), sizeof p->fgroup);
+            memcpy(p->rgroup, pwcache_get_group(p->rgid), sizeof p->rgroup);
+            memcpy(p->sgroup, pwcache_get_group(p->sgid), sizeof p->sgroup);
+            memcpy(p->fgroup, pwcache_get_group(p->fgid), sizeof p->fgroup);
         }
     }
 
@@ -962,41 +979,34 @@ static proc_t* simple_readproc(PROCTAB *restrict const PT, proc_t *restrict cons
             fill_environ_cvt(path, p);
         else
             p->environ = file2strvec(path, "environ");
-    } else
-        p->environ = NULL;
+    }
 
     if (flags & (PROC_FILLCOM|PROC_FILLARG)) {  // read /proc/#/cmdline
         if (flags & PROC_EDITCMDLCVT)
             fill_cmdline_cvt(path, p);
         else
             p->cmdline = file2strvec(path, "cmdline");
-    } else
-        p->cmdline = NULL;
+    }
 
     if ((flags & PROC_FILLCGROUP)) {            // read /proc/#/cgroup
         if (flags & PROC_EDITCGRPCVT)
             fill_cgroup_cvt(path, p);
         else
             p->cgroup = file2strvec(path, "cgroup");
-    } else
-        p->cgroup = NULL;
+    }
 
-#ifdef OOMEM_ENABLE
     if (unlikely(flags & PROC_FILLOOM)) {
         if (likely(file2str(path, "oom_score", &ub) != -1))
             oomscore2proc(ub.buf, p);
         if (likely(file2str(path, "oom_adj", &ub) != -1))
             oomadj2proc(ub.buf, p);
     }
-#endif
 
     if (unlikely(flags & PROC_FILLNS))          // read /proc/#/ns/*
         ns2proc(path, p);
 
-#ifdef WITH_SYSTEMD
     if (unlikely(flags & PROC_FILLSYSTEMD))     // get sd-login.h stuff
         sd2proc(p);
-#endif
 
     if (unlikely(flags & PROC_FILL_LXC))        // value the lxc name
         p->lxcname = lxc_containers(path);
@@ -1053,21 +1063,21 @@ static proc_t* simple_readtask(PROCTAB *restrict const PT, const proc_t *restric
 
     /* some number->text resolving which is time consuming */
     if (flags & PROC_FILLUSR){
-        memcpy(t->euser, user_from_uid(t->euid), sizeof t->euser);
+        memcpy(t->euser, pwcache_get_user(t->euid), sizeof t->euser);
         if(flags & PROC_FILLSTATUS) {
-            memcpy(t->ruser, user_from_uid(t->ruid), sizeof t->ruser);
-            memcpy(t->suser, user_from_uid(t->suid), sizeof t->suser);
-            memcpy(t->fuser, user_from_uid(t->fuid), sizeof t->fuser);
+            memcpy(t->ruser, pwcache_get_user(t->ruid), sizeof t->ruser);
+            memcpy(t->suser, pwcache_get_user(t->suid), sizeof t->suser);
+            memcpy(t->fuser, pwcache_get_user(t->fuid), sizeof t->fuser);
         }
     }
 
     /* some number->text resolving which is time consuming */
     if (flags & PROC_FILLGRP){
-        memcpy(t->egroup, group_from_gid(t->egid), sizeof t->egroup);
+        memcpy(t->egroup, pwcache_get_group(t->egid), sizeof t->egroup);
         if(flags & PROC_FILLSTATUS) {
-            memcpy(t->rgroup, group_from_gid(t->rgid), sizeof t->rgroup);
-            memcpy(t->sgroup, group_from_gid(t->sgid), sizeof t->sgroup);
-            memcpy(t->fgroup, group_from_gid(t->fgid), sizeof t->fgroup);
+            memcpy(t->rgroup, pwcache_get_group(t->rgid), sizeof t->rgroup);
+            memcpy(t->sgroup, pwcache_get_group(t->sgid), sizeof t->sgroup);
+            memcpy(t->fgroup, pwcache_get_group(t->fgid), sizeof t->fgroup);
         }
     }
 
@@ -1085,29 +1095,24 @@ static proc_t* simple_readtask(PROCTAB *restrict const PT, const proc_t *restric
                 fill_environ_cvt(path, t);
             else
                 t->environ = file2strvec(path, "environ");
-        } else
-            t->environ = NULL;
+        }
 
         if (flags & (PROC_FILLCOM|PROC_FILLARG)) {      // read /proc/#/task/#/cmdline
             if (flags & PROC_EDITCMDLCVT)
                 fill_cmdline_cvt(path, t);
             else
                 t->cmdline = file2strvec(path, "cmdline");
-        } else
-            t->cmdline = NULL;
+        }
 
         if ((flags & PROC_FILLCGROUP)) {                // read /proc/#/task/#/cgroup
             if (flags & PROC_EDITCGRPCVT)
                 fill_cgroup_cvt(path, t);
             else
                 t->cgroup = file2strvec(path, "cgroup");
-        } else
-            t->cgroup = NULL;
+        }
 
-#ifdef WITH_SYSTEMD
         if (unlikely(flags & PROC_FILLSYSTEMD))         // get sd-login.h stuff
             sd2proc(t);
-#endif
 
         if (unlikely(flags & PROC_FILL_LXC))            // value the lxc name
             t->lxcname = lxc_containers(path);
@@ -1123,11 +1128,11 @@ static proc_t* simple_readtask(PROCTAB *restrict const PT, const proc_t *restric
         t->dt       = p->dt;
         t->cmdline  = p->cmdline;  // better not free these until done with all threads!
         t->environ  = p->environ;
+        t->cgname   = p->cgname;
         t->cgroup   = p->cgroup;
         if (t->supgid) free(t->supgid);
         t->supgid   = p->supgid;
         t->supgrp   = p->supgrp;
-#ifdef WITH_SYSTEMD
         t->sd_mach  = p->sd_mach;
         t->sd_ouid  = p->sd_ouid;
         t->sd_seat  = p->sd_seat;
@@ -1135,20 +1140,17 @@ static proc_t* simple_readtask(PROCTAB *restrict const PT, const proc_t *restric
         t->sd_slice = p->sd_slice;
         t->sd_unit  = p->sd_unit;
         t->sd_uunit = p->sd_uunit;
-#endif
         t->lxcname = p->lxcname;
         MK_THREAD(t);
     }
 #endif
 
-#ifdef OOMEM_ENABLE
     if (unlikely(flags & PROC_FILLOOM)) {
         if (likely(file2str(path, "oom_score", &ub) != -1))
             oomscore2proc(ub.buf, t);
         if (likely(file2str(path, "oom_adj", &ub) != -1))
             oomadj2proc(ub.buf, t);
     }
-#endif
 
     if (unlikely(flags & PROC_FILLNS))                  // read /proc/#/task/#/ns/*
         ns2proc(path, t);
@@ -1169,7 +1171,7 @@ static int simple_nextpid(PROCTAB *restrict const PT, proc_t *restrict const p)
   char *restrict const path = PT->path;
   for (;;) {
     ent = readdir(PT->procfs);
-    if(unlikely(unlikely(!ent) || unlikely(!ent->d_name))) return 0;
+    if(unlikely(unlikely(!ent) || unlikely(!ent->d_name[0]))) return 0;
     if(likely(likely(*ent->d_name > '0') && likely(*ent->d_name <= '9'))) break;
   }
   p->tgid = strtoul(ent->d_name, NULL, 10);
@@ -1196,7 +1198,7 @@ static int simple_nexttid(PROCTAB *restrict const PT, const proc_t *restrict con
   }
   for (;;) {
     ent = readdir(PT->taskdir);
-    if(unlikely(unlikely(!ent) || unlikely(!ent->d_name))) return 0;
+    if(unlikely(unlikely(!ent) || unlikely(!ent->d_name[0]))) return 0;
     if(likely(likely(*ent->d_name > '0') && likely(*ent->d_name <= '9'))) break;
   }
   t->tid = strtoul(ent->d_name, NULL, 10);
